<?php
/**
 * Post type declaration file.
 *
 * @package Super Simple Slider/Includes
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Post type declaration class.
 */
class Super_Simple_Form_Control {
	
	/**
	 * The main plugin object.
	 *
	 * @var     object
	 * @access  public
	 * @since   1.0.0
	 */
	public $parent = null;
		
    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $id;
    
    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $type;

    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $name;
		    
    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $placeholder;

    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $prefix;
    
    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $suffix;
    
    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $label;

    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $description;
    
    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $value;

    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $show_labels;
    
    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $field_counter;
    
    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $template;

    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $fieldset;

    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $options;

    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $input_attrs;
    
    /**
     * Control ID.
     *
     * @since 3.4.0
     * @var string
     */
    public $class;
    
	public function __construct( $id, $parent, $repeatable, $args = array(), $value, $field_counter = '' ) {
		$keys = array_keys( get_object_vars( $this ) );
		
        foreach ( $keys as $key ) {
            if ( isset( $args[ $key ] ) ) {
                $this->$key = $args[ $key ];
            }
        }
        
		$this->id = $id;
		$this->parent = $parent;
		$this->repeatable = $repeatable;
		$this->value = $value;
		$this->field_counter = $field_counter;
		
		$this->render();
	}
	
    /**
     * Renders the control wrapper and calls $this->render_content() for the internals.
     *
     * @since 3.4.0
     */
    protected function render() {
        $id    = 'customize-control-' . str_replace( array( '[', ']' ), array( '-', '' ), $this->id );
        $class = 'customize-control customize-control-' . $this->type;
 
        //printf( '<li id="%s" class="%s">', esc_attr( $id ), esc_attr( $class ) );
        $this->render_content();
        //echo '</li>';
    }
    
	public function render_content() {
		$html 		= '';
		$field_name = '';
		
		$field_name  .= $this->id;
        $field_class = 'otb-form-control otb-form-control-' . $this->type . ' ' . $this->id . ' ' . $this->class;

		if ( $this->field_counter ) {
			$field_name .= $this->field_counter;
		}
        
		if ( $this->repeatable ) {
			$field_name .= '[]';
		}
		
		if ( $this->label ) {
			//$html .= '<h4>' . $this->label . '</h4>';
			$html .= '<label for="' . $this->id . '">' . $this->label . '</label>';
		}
		
		switch( $this->type ) {

			case 'text':
				$html .= '<input type="text" name="' . esc_attr( $field_name ) . '" class="' . esc_attr( $field_class ) . '" placeholder="' . esc_attr( $this->placeholder ) . '" value="' . esc_html( $this->value ) . '" />' . "\n";
			break;

			case 'email':
				$html .= '<input id="' . esc_attr( $this->id ) . '" type="email" name="' . esc_attr( $field_name ) . '" class="' . esc_attr( $field_class ) . '" placeholder="' . esc_attr( $this->placeholder ) . '" value="' . esc_html( $this->value ) . '" />' . "\n";
			break;
			
			case 'tel':
				$html .= '<input id="' . esc_attr( $this->id ) . '" type="tel" name="' . esc_attr( $field_name ) . '" class="' . esc_attr( $field_class ) . '" placeholder="' . esc_attr( $this->placeholder ) . '" value="' . esc_html( $this->value ) . '" />' . "\n";
			break;
			
			case 'number':
			case 'float':
				if ( $this->prefix || $this->suffix ) {
					$input_group_classes = array();
					
					if ( $this->prefix ) {
						$input_group_classes[] = 'prefix';
					}

					if ( $this->suffix ) {
						$input_group_classes[] = 'suffix';
					}
					
					$html .= '<div class="input-group ' .esc_attr( implode( ' ', $input_group_classes ) ). '">';
				}

				if ( $this->prefix ) {
					$html .= '<span class="input-group-addon prefix">' . $this->prefix . '</span>';
				}
				
				$html .= '<input id="' . esc_attr( $this->id ) . '" type="number" name="' . esc_attr( $field_name ) . '" class="' . esc_attr( $field_class ) . '" placeholder="' . esc_attr( $this->placeholder ) . '" value="' . esc_html( $this->value ) . '" />' . "\n";

				if ( !empty( $this->suffix ) ) {
					$html .= '<span class="input-group-addon suffix">' . $this->suffix . '</span>';
				}

				if ( $this->prefix || $this->suffix ) {
					$html .= '</div>';
				}
			break;

			case 'url':
				if ( $this->prefix ) {
					$html .= '<div class="input-group prefix">';
					$html .= '<span class="input-group-addon ">' . $this->prefix . '</span>';
				}
				
				$html .= '<input id="' . esc_attr( $this->id ) . '" type="url" name="' . esc_attr( $field_name ) . '" class="' . esc_attr( $field_class ) . '" placeholder="' . esc_attr( $this->placeholder ) . '" value="' . esc_html( $this->value ) . '" />' . "\n";
				
				if ( !empty( $this->prefix ) ) {
					$html .= '</div>';
				}
			break;
			
			case 'password':
				$html .= '<input id="' . esc_attr( $this->id ) . '" type="password" name="' . esc_attr( $field_name ) . '" class="' . esc_attr( $field_class ) . '" placeholder="' . esc_attr( $this->placeholder ) . '" value="' . esc_html( $this->value ) . '" />' . "\n";
			break;
			
			case 'textarea':
				$html .= '<textarea id="' . esc_attr( $this->id ) . '" placeholder="' . esc_attr( $this->placeholder ) . '" class="' . esc_attr( $field_class ) . '" cols="55" rows="5" name="' . esc_attr( $field_name ) . '">' . esc_html( $this->value ) . '</textarea>' . "\n";
			break;
			
			case 'tinymce':
				$settings = array(
				    'teeny' => false,
				    'textarea_name' => $field_name,
				    'textarea_rows' => 5,
					'tabindex' => 1
				);
				
				$html .= wp_editor( $this->value, $this->id .'-'. uniqid(), $settings );
			break;

			case 'checkbox':
				$checked = '';
				if ( $this->value ) {
					$checked = 'checked="checked"';
				}

				$html .= '<input id="' . esc_attr( $this->id ) . '" type="checkbox" name="' . esc_attr( $field_name ) . '_checkbox" ' . $checked . ' class="' . esc_attr( $field_class ) . '"/>';
				$html .= '<input type="hidden" name="' . esc_attr( $field_name ) . '" value="' . esc_html( $this->value ) . '" />';
			break;
			
			case 'select':
				$html .= '<select name="' . esc_attr( $field_name ) . '" class="' . esc_attr( $field_class ) . '">';
				foreach ( $this->options as $k => $v ) {
					$selected = false;
					if ( $k == $this->value ) {
						$selected = true;
					}
					$html .= '<option ' . selected( $selected, true, false ) . ' value="' . esc_attr( $k ) . '">' . $v . '</option>';
				}
				$html .= '</select> ';
			break;

			case 'milliseconds':
				$html .= '<input id="' . esc_attr( $this->id ) . '" type="text" name="' . esc_attr( $field_name ) . '" class="' . esc_attr( $field_class ) . '" placeholder="' . esc_attr( $this->placeholder ) . '" value="' . esc_html( $this->value ) . '" /> ms' . "\n";
			break;

			case 'percentage':
				$html .= '<input id="' . esc_attr( $this->id ) . '" type="text" name="' . esc_attr( $field_name ) . '" class="' . esc_attr( $field_class ) . '" placeholder="' . esc_attr( $this->placeholder ) . '" value="' . esc_html( $this->value ) . '" /> %' . "\n";
			break;

			case 'color':
				$html .= '<input id="' . esc_attr( $this->id ) . '" type="text" name="' . esc_attr( $field_name ) . '" class="colorpicker ' . esc_attr( $field_class ) . '" maxlength="7" value="' .esc_attr( $this->value ). '" />' . "\n";
			break;

			case 'range':
				$html .= '<input id="' . esc_attr( $this->id ) . '" type="range" name="' . esc_attr( $field_name ) . '" class="' . esc_attr( $field_class ) . '" min="' . esc_attr( $this->input_attrs['min'] ) . '" max="' . esc_attr( $this->input_attrs['max'] ) . '" step="' . esc_attr( $this->input_attrs['step'] ) . '" value="' . esc_attr( $this->value ) . '" />' . "\n";
				
				if ( $this->show_labels ) {
					$html .= '<ul class="labels">';
					
					for ( $i=$this->input_attrs['min']; $i<$this->input_attrs['max']; $i+=$this->input_attrs['step']) {
						$html .= '<li>' .$i. '</li>';
					}
					
					$html .= '</ul>';
				}
			break;

			case 'media_upload':
				global $_wp_additional_image_sizes;
				//$image_id 	   = get_post_meta( $post->ID, 'header_image_id', true );
				$image_id 		   = $this->value;
				$has_image		   	  = false;
				$media_uploader_class = '';
				
				if ( $image_id && get_post( $image_id ) ) {
					$has_image = true;
					$media_uploader_class = 'has-img';
				}
				
				$html .= '<div class="media-uploader ' .$media_uploader_class. '">';
				$html .= '<p class="hide-if-no-js"><a title="' . esc_attr__( 'Upload Image', 'super-simple-slider' ) . '" href="javascript:;" class="button upload" data-uploader_title="' . esc_attr__( 'Upload Image', 'super-simple-slider' ) . '" data-uploader_button_text="' . esc_attr__( 'Upload Image', 'super-simple-slider' ) . '">' . esc_html__( 'Upload Image', 'super-simple-slider' ) . '</a></p>';				
				$html .= '<input type="hidden" name="' . esc_attr( $field_name ) . '" value="' . esc_attr( $image_id ) . '" />';
				
				$html .= '<div class="preview">';
				$html .= '<div class="delete icon">';
				$html .= '<i class="sss-fa sss-fa-times"></i>';		
				$html .= '</div>';
				
				if ( $has_image ) {
					$thumbnail_html = wp_get_attachment_image( $image_id, 'full' );

					if ( !empty( $thumbnail_html ) ) {
						$html .= $thumbnail_html;
					}
				
				} else {
					$html .= '<img src="" />';
					//$html .= '<p class="hide-if-no-js"><a title="' . esc_attr__( 'Upload Image', 'super-simple-slider' ) . '" href="javascript:;" id="upload_header_image_button" class="button upload" data-uploader_title="' . esc_attr__( 'Upload Image', 'super-simple-slider' ) . '" data-uploader_button_text="' . esc_attr__( 'Upload Image', 'super-simple-slider' ) . '">' . esc_html__( 'Upload Image', 'super-simple-slider' ) . '</a></p>';
					//$html .= '<input type="hidden" id="upload_header_image" name="' . esc_attr( $field_name ) . '" value="" />';
				}
				
				$html .= '</div>';
				$html .= '</div>';
				$html .= '</div>';
			break;
			
			case 'dropdown_pages_posts':
				$html = '<select name="' . esc_attr( $field_name ) . '" class="' . esc_attr( $field_class ) . '" data-customize-setting-link="' .$this->id. '">';
				$html .= '<option value="">Not linked</option>';
				$html .= '<option ' . selected( $this->value, 'custom', false ) . ' value="custom">Custom URL</option>';
				$html .= '<optgroup label="Pages">';
				
				// Get all published pages
				$pages = get_pages();
				foreach ($pages as $page) {
					$html .= '<option ' . selected( $this->value, $page->ID, false ) . ' value="' .$page->ID. '">' .$page->post_title. '</option>';
				}
				
				// Prevent weirdness
				wp_reset_postdata();
				
				$html .= '</optgroup>';
				$html .= '<optgroup label="Posts">';
		
				// Get all published posts
				$posts = get_posts( array( 'posts_per_page'   => -1 ) );
				foreach ($posts as $post) {
					$html .= '<option ' . selected( $this->value, $post->ID, false ) . ' value="' .$post->ID. '">' .$post->post_title. '</option>';
				}
				
				// Prevent weirdness
				wp_reset_postdata();
				
				$html .= '</optgroup>';
				$html .= '</select>';		
			
			break;
			
			case 'repeatable_fieldset':
				ob_start();
				include ( super_simple_slider()->assets_dir .'/template-parts/'. $this->template );
				$html .= ob_get_clean();
			break;
		}
		
		if ( $this->description ) {
			$html .= '<div class="description">' . $this->description . '</div>';
		}
		
		echo $html;
	}
}
